import java.util.*;
import jas.engine.Sim;

/**
 * The OrderGenerator is used at present to simulate the market
 * behavior.
 *
 * @author Pietro Terna 
 * @author Michele Sonnessa: JAS porting  
 * */

public class OrderGenerator  
{

	/** How many unit we have, used at present to define the max value
			used to generate the 'digits' composing our recipes */
	
	/** the max length of a recipe */
	public int maxStepNumber;
	
	/** the max length of a step in a recipe */
	public int maxStepLength;
	
	/** used to record the number of generated orders*/
	public int orderCount;
	
	/** a specific order*/
	public Order anOrder;
	/** the list containig the operating units*/
	public ArrayList unitList;
	/** the model (to which we apply mark up of the order revenue */
	ESFrameModel eSFrameModel;
	
	/** the integer vector containing a specific production recipe */
	public int[] orderRecipe1;
	
	/** the integer vector containing the unit of time spent for each step
	 *  in the recipe; il will determine the tranformation of the orderRecipe1
	 *  vector into the orderRecipe2 vector containing the repetitions
	 *  of the steps, to reproduce the length of each step in time
	 *
	 *  NB this formalism is internal to internal (in external formalism
	 *  we have 'step s/h/d time'
	 *
	 *  as an examples
	 *
	 *  orderRecipe1      contains  1 12  7  3
	 *  stepLengthInOrder contains  1  3  2  2
	 *
	 *  orderRecipe2      contains  1 12 12 12  7  7  3  3
	 */
	public int[] stepLengthInOrder;
	
	/** the integer vector containing a specific production recipe,
	 *  developed following the length of each step
	 */
	public int[] orderRecipe2;
	
	/** the dictionary to be used to choose the steps to be included in
	    a recipe */
	public int[] dictionary;
	/** length of the dictionary */
	public int dictionaryLength;
	
	/** the tool used in assigning orders */
	AssigningTool assigningTool;
	
	// units
	Unit aUnit;
	
	/**
	 * the constructor for OrderGenerator
	 */
	public OrderGenerator (int msn, int msl, ArrayList ul,
			   											ESFrameModel mo, AssigningTool at)
	{
	
		// Call the constructor for the OrderGenerator parent class.
		int dim;
		
		orderCount=0;
		maxStepNumber=msn;
		if(maxStepNumber==0)
		{
		    System.out.println("maxStepNumber cannot be 0 in this run.");
		    System.exit(1);
		}
		maxStepLength=msl;
		if(maxStepLength==0)
		{
		    System.out.println("maxStepLength cannot be 0 in this run.");
		    System.exit(1);
		}
		unitList=ul;
		
		orderRecipe1 = new int[maxStepNumber];
		stepLengthInOrder = new int[maxStepNumber];
		dim=maxStepNumber*maxStepLength;
		orderRecipe2 = new int[dim];
		
		eSFrameModel = mo;
		assigningTool=at;
	}

	/** building the dictionary containing the steps to be included in the
	recipes */
	public void setDictionary()
	{
		int i, ii;
		// how many words in the dictionary?
		
		// starting point: the dictionary length
		dictionaryLength=unitList.size();

		if(ESFrameModel.verbose)
		    System.out.println(
		 "Length of the dictionary (# of codes identifying steps in recipes): "
		 + dictionaryLength);

		//creating the dictionary
		dictionary = new int[dictionaryLength];
		
		// loading terms into the dictionary
		i=0;
		for (ii=0;ii<unitList.size();ii++)
		    {
			aUnit=(Unit) unitList.get(ii);
			dictionary[i++]=aUnit.getProductionPhase();
		    }
		
		if(ESFrameModel.verbose)
		    for (i=0;i<dictionaryLength;i++)
			System.out.println(i + "  " + dictionary[i]);
		}
		
		
		/** creating an order form, choosing its recipe length and content */
		public void createRandomOrderWithNSteps() {
		int i, ii, iii, k, count, kk, n,
		    randomStepNumber, randomStepNumber0, randomStep;
		
		orderCount++;
		count=0;
		
		if(unitList.size()==0){
		    System.out.println("totalUnitNumber cannot be 0.");
		    System.exit(1);}
		
		
		// ------------------------------
		// normal form creation process
		randomStepNumber=Sim.getRnd().getIntFromTo (1, maxStepNumber);
		
		// generating the sequence in the recipe
		
		// putting 0 in the vector may be useful for future software
		// modifications
		for (i=0;i<maxStepNumber;i++)orderRecipe1[i]=0;
		for (i=0;i<maxStepNumber;i++)stepLengthInOrder[i]=0;
		
		randomStepNumber0=0;
		
		for (i=0;i<randomStepNumber;i++)
		    {
			randomStepNumber0=i+1;
		
			randomStep = Sim.getRnd().getIntFromTo (0,
						      			dictionaryLength-1);
			orderRecipe1[i]= dictionary[randomStep];
		
			// generating the length of each step
			if (maxStepLength>1)
			    stepLengthInOrder[i]=
				Sim.getRnd().getIntFromTo
				(1, maxStepLength);
			else
			    stepLengthInOrder[i]=1;
		    }
		
		// putting together
		count=0;
		for (i=0;i<randomStepNumber0;i++)
		    for (ii=0;ii<stepLengthInOrder[i];ii++)
			{
			    // copying normal steps
			    orderRecipe2[count++]=orderRecipe1[i];
		
			}
		
		if(ESFrameModel.verbose)
		{
		    for (iii=0; iii<count;iii++)
			System.out.print(orderRecipe2[iii]+ " ");
		    System.out.println(" internal recipe for order " + orderCount);
		}

		// ------------------------------
		// concluding normal order form generation
		
		// creating an order
		    // vector dimension in anOrder is count: count is a step ahead (see
		    // last count++ operation) but starting with 0 is the correct number
		    // of elements
		anOrder = new Order(orderCount,
				    Sim.getAbsoluteTime(),
				    count, orderRecipe2,
				    eSFrameModel);
		
		if(ESFrameModel.verbose)
		{
		    for (iii=0; iii<anOrder.orderRecipe.length;iii++)
			System.out.print(anOrder.orderRecipe[iii]+ " ");
		    System.out.println(" internal recipe into the order " +
				       orderCount);
		}


		/**
		 * sending the order to the first production unit
		 * (we are acting as the Front End of the simulated enterprise)
		 */
		assigningTool.assign(anOrder);


	}

}
